/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.testsuites;

import java.awt.AWTException;
import java.io.IOException;
import java.util.List;



import org.thermostat.qa.framework.Assert;
import org.thermostat.qa.framework.ThermostatTest;
import org.thermostat.qa.annotations.StorageType;
import org.thermostat.qa.annotations.StorageTypes;

/**
 * Class AgentTest
 * contains basic smoke tests for starting the Thermostat agent. Starting the agent without
 * various parts of web storage (which should not work) and with web storage is checked.
 *
 */
@StorageType(StorageTypes.WEBTOMCAT_STORAGE)
public class AgentWebStorageTest extends ThermostatTest
{
    private String[] THERM_AGENT_WITH_TOMCAT_ONLY_THERMOSTAT_LOGS;
    private String[] THERM_AGENT_WITH_TOMCAT_ONLY_CATALINA_LOGS;
    private String[] THERM_AGENT_WITH_WEB_STORAGE_THERMOSTAT_LOGS;
    private String[] THERM_AGENT_WITH_WEB_STORAGE_CATALINA_LOGS;
    private String[] THERM_AGENT_BAD_LOGIN_CATALINA_LOGS;
    
    protected void setCheckedStrings()
    {
        THERM_AGENT_WITH_TOMCAT_ONLY_THERMOSTAT_LOGS = new String[] {
            "com.mongodb.MongoException.Network: Read operation to server /127.0.0.1:27518 failed on database thermostat",
            "java.net.ConnectException: Connection refused"
            };
        THERM_AGENT_WITH_TOMCAT_ONLY_CATALINA_LOGS = new String[] {
            "WebStorageEndPoint: Initializing web service",
            "WebStorageEndPoint: THERMOSTAT_HOME == " + this.configuration.getThermostatHome(),
            "WARNING - MongoConnection: Failed to connect to storage",
            "com.mongodb.MongoException.Network: Read operation to server /127.0.0.1:27518 failed on database thermostat",
            "java.net.ConnectException: Connection refused"
        };
       
        THERM_AGENT_WITH_WEB_STORAGE_THERMOSTAT_LOGS = new String[] {
            "WebStorageEndPoint: Initializing web service",
            "WebStorageEndPoint: THERMOSTAT_HOME == " + this.configuration.getThermostatHome(),
            "MongoConnection: Using plain socket for mongodb://",
            "WebStorageEndPoint: (id: 0) registered non-aggreate category: vm-cpu-stats|com.redhat.thermostat.vm.cpu.common.model.VmCpuStat|{timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, cpuLoad=Key: cpuLoad}",
            "WebStorageEndPoint: (id: 1) registered non-aggreate category: vm-heap-info|com.redhat.thermostat.vm.heap.analysis.common.model.HeapInfo|{timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, heapId=Key: heapId, heapDumpId=Key: heapDumpId, histogramId=Key: histogramId}",
            "WebStorageEndPoint: (id: 2) registered non-aggreate category: vm-gc-stats|com.redhat.thermostat.vm.gc.common.model.VmGcStat|{runCount=Key: runCount, timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, wallTime=Key: wallTime, collectorName=Key: collectorName}",
            "WebStorageEndPoint: (id: 3) registered non-aggreate category: vm-memory-stats|com.redhat.thermostat.vm.memory.common.model.VmMemoryStat|{timeStamp=Key: timeStamp, generations=Key: generations, agentId=Key: agentId, vmId=Key: vmId}",
            "WebStorageEndPoint: (id: 4) registered non-aggreate category: numa-stat|com.redhat.thermostat.numa.common.NumaStat|{nodeStats=Key: nodeStats, timeStamp=Key: timeStamp, agentId=Key: agentId}",
            "WebStorageEndPoint: (id: 5) registered non-aggreate category: numa-host-info|com.redhat.thermostat.numa.common.NumaHostInfo|{agentId=Key: agentId, hostNumNumaNodes=Key: hostNumNumaNodes}",
            "WebStorageEndPoint: (id: 6) registered non-aggreate category: memory-stats|com.redhat.thermostat.host.memory.common.model.MemoryStat|{total=Key: total, cached=Key: cached, free=Key: free, timeStamp=Key: timeStamp, swapFree=Key: swapFree, swapTotal=Key: swapTotal, buffers=Key: buffers, agentId=Key: agentId, commitLimit=Key: commitLimit}"
            };
        
        THERM_AGENT_WITH_WEB_STORAGE_CATALINA_LOGS = new String[] {
            "INFO: Deploying web application directory",
            "/webapps/thermostat",
            "WebStorageEndPoint: Initializing web service",
            "WebStorageEndPoint: THERMOSTAT_HOME == " + this.configuration.getThermostatHome(),
            "MongoConnection: Using plain socket for mongodb://",
            "WebStorageEndPoint: (id: 0) registered non-aggreate category: vm-cpu-stats|com.redhat.thermostat.vm.cpu.common.model.VmCpuStat|{timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, cpuLoad=Key: cpuLoad}",
            "WebStorageEndPoint: (id: 1) registered non-aggreate category: vm-heap-info|com.redhat.thermostat.vm.heap.analysis.common.model.HeapInfo|{timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, heapId=Key: heapId, heapDumpId=Key: heapDumpId, histogramId=Key: histogramId}",
            "WebStorageEndPoint: (id: 2) registered non-aggreate category: vm-gc-stats|com.redhat.thermostat.vm.gc.common.model.VmGcStat|{runCount=Key: runCount, timeStamp=Key: timeStamp, agentId=Key: agentId, vmId=Key: vmId, wallTime=Key: wallTime, collectorName=Key: collectorName}",
            "WebStorageEndPoint: (id: 3) registered non-aggreate category: vm-memory-stats|com.redhat.thermostat.vm.memory.common.model.VmMemoryStat|{timeStamp=Key: timeStamp, generations=Key: generations, agentId=Key: agentId, vmId=Key: vmId}",
            "WebStorageEndPoint: (id: 4) registered non-aggreate category: numa-stat|com.redhat.thermostat.numa.common.NumaStat|{nodeStats=Key: nodeStats, timeStamp=Key: timeStamp, agentId=Key: agentId}"
            };
        
        THERM_AGENT_BAD_LOGIN_CATALINA_LOGS = new String[] {
            "WARNING: Login exception authenticating username \"doctor-evil\"",
            "javax.security.auth.login.LoginException: User 'doctor-evil' not found"
        };
    }
    
    protected List<String> listAgents() throws IOException
    {
        logInfo("lists agents", "if any");
        List<String> output = runHelperBashScript("list_all_therm_agents.sh");
        for (String message : output)
        {
            logInfo("Agent list", message);
        }    
        return output;
    }
    
    @Override
    protected void tearDown()
    {
        try
        {
            restoreNormalStorageConfigFiles();
            runHelperBashScript("stop_all_therm_agents.sh");
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }      
    }

    @Override
    protected void setUp()
    {
        setCheckedStrings();
        try
        {
            eraseTomcatOutputFiles();
            prepareWebStorageConfigFiles();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
    }

    /**
     * Method testStartWithoutStorage
     * tries to start thermostat agent in new thread without previously
     * starting the storage or tomcat. There should be no thermostat agents
     * running.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     * @throws InterruptedException 
     */
    public void testStartWithoutAnyStorage() throws IOException, InterruptedException
    {
        //start only agent
        Thread agent = new AgentThread();
        agent.start();
        logInfo("runAgentThread", "Thermostat Agent tried to start");
        
        //wait before the agent realizes it cannot run and stops 
        agent.join(FIVE_SEC);
      
        //there should not be any agent running (check the jps)
        List<String> output = listAgents();
       
        //to be sure there is no running agent
        runHelperBashScript("stop_all_therm_agents.sh");
        
        Assert.assertTrue(output.size()==0, "there should not be any running thermostat agents");
    }
    
    /**
     * Method testStartWithTomcatWithoutStorage
     * tries to start thermostat agent in new thread without previously
     * starting the storage, but with running tomcat. There may be an agent
     * running, because it connects to the web storage not to the db, but
     * there should definitely be some logs in the tomcat logs directory
     * about not being able to connect to the underlying mongodb.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     */
    public void testStartWithTomcatWithoutStorage() throws IOException 
    {
        startTomcat();
        
        //start only agent
        new AgentThread().start();
        logInfo("runAgentThread", "Thermostat Agent tried to start");
        
        //wait before the agent is stopped, 
        //so it can communicate with web storage
        sleep(3000);
      
        //switch off the agent that probably runs, because tomcat was on
        runHelperBashScript("stop_all_therm_agents.sh");
 
        stopTomcat();
        sleep(FIVE_SEC);

        stopTomcat();
        sleep(FIVE_SEC);
        
        //analyse the contents of logs in tomcat
        createTomcatThermostatOutputFiles();
        
        List<String> outputs = getTomcatOutputs("catalina-logs.txt");
        checkPresenceOfPatterns(outputs, THERM_AGENT_WITH_TOMCAT_ONLY_CATALINA_LOGS);
               
        outputs = getTomcatOutputs("thermostat-web-storage-logs.txt");
        checkPresenceOfPatterns(outputs, THERM_AGENT_WITH_TOMCAT_ONLY_THERMOSTAT_LOGS);
                
        outputs = getTomcatOutputs("localhost_access_logs.txt");
        
        listAgents();
    }
    
    /**
     * Method testStartWithStorageWithoutTomcat
     * tries to start thermostat agent in new thread without previously
     * starting tomcat, only the db is running. There should be no thermostat
     * agents running, because the agent connects to web storage.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     * @throws InterruptedException 
     */
    public void testStartWithStorageWithoutTomcat() throws IOException, InterruptedException
    {
        startStorage();
        
        //start only agent
        Thread agent = new AgentThread();
        agent.start();
        logInfo("runAgentThread", "Thermostat Agent tried to start");
        
        //wait till agent realizes it cannot run and stops
        agent.join(6000);
        
        //there should not be any agent running (check the jps)
        List<String> output = runHelperBashScript("list_all_therm_agents.sh");

        //to be sure there are no running agents
        runHelperBashScript("stop_all_therm_agents.sh");

        stopStorage();

        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");
        for (String message : output)
        {
            logInfo("Agent list", message);
        }
        Assert.assertTrue(output.size()==0, "there should not be any running thermostat agents");
    }
    
    /**
     * Method testStartAgent
     * starts the storage and then tries to start thermostat agent in a new
     * thread. There should be one thermostat agent running.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     * @throws AWTException 
     */
    public void testStartAgent() throws IOException, AWTException
    {
        startWebStorage();
        
        //start agent
        new AgentThread().start();
        logInfo("runAgentThread", "Thermostat Agent tried to start");
        
        //check if the agent is running (jps)
        List<String> output = listAgents();
        Assert.assertNotNull(output, "error running script 'list_all_therm_agents.sh'");
        Assert.assertTrue(output.size()>0, "there should be >0 running thermostat agents");

        //let the agent write some data
        sleep(3000);
        
        //stop the agent
        output = runHelperBashScript("stop_all_therm_agents.sh");
        Assert.assertNotNull(output, "error running script 'stop_all_therm_agents.sh'");
        
        stopWebStorage();
        
        sleep(3000);
        
      //analyse the contents of logs in tomcat
        createTomcatThermostatOutputFiles();
        
        List<String> outputs = getTomcatOutputs("catalina-logs.txt");
                
        checkPresenceOfPatterns(outputs,THERM_AGENT_WITH_WEB_STORAGE_CATALINA_LOGS);
        
        outputs = getTomcatOutputs("thermostat-web-storage-logs.txt");
                
        checkPresenceOfPatterns(outputs,THERM_AGENT_WITH_WEB_STORAGE_THERMOSTAT_LOGS);
        
        outputs = getTomcatOutputs("localhost_access_logs.txt");        
    }
    
    /**
     * Method testStartAgentWithBadLogin
     * starts the web storage and tries to connect an agent with bad login.
     * The attempt should be logged in catalina.out as unsuccessful login.
     * 
     * This test uses the following scripts:
     * <ul>
     * <li><a href="scripts/list_all_therm_agents.sh">list_all_therm_agents.sh</a>,
     * <li><a href="scripts/stop_all_therm_agents.sh">stop_all_therm_agents.sh</a>.
     * <ul>
     * 
     * @throws IOException
     */
    public void testStartAgentWithBadLogin() throws IOException{
        //copy bad login info into agent conf
        prepareWebStorageBadAgentLoginFile();
        //try to run
        startWebStorage();
        
        //start agent
        new AgentThread().start();
        logInfo("runAgentThread", "Thermostat Agent tried to start");
        
        //check if the agent is running (jps)
        List<String> outputs = listAgents();
        Assert.assertNotNull(outputs, "error running script 'list_all_therm_agents.sh'");
        Assert.assertTrue(outputs.size()>0, "there should be a running thermostat agent trying to access the db");

        //let the agent try to use its login for web storage
        sleep(6000);
        
        //stop the agent
        outputs = runHelperBashScript("stop_all_therm_agents.sh");
        Assert.assertNotNull(outputs, "error running script 'stop_all_therm_agents.sh'");
                
        stopWebStorage();
        sleep(3000);
         
        //analyse the contents of logs in tomcat
        createTomcatThermostatOutputFiles();
        
        outputs = getTomcatOutputs("catalina-logs.txt");
        /*for(String s : outputs)
        {
            logInfo("Bad login catalina-logs.txt: ", s);
        }*/
        checkPresenceOfPatterns(outputs, THERM_AGENT_BAD_LOGIN_CATALINA_LOGS);
        
        outputs = getTomcatOutputs("thermostat-web-storage-logs.txt");
        /*for(String s : outputs)
        {
            logInfo("Bad login thermostat-web-storage-logs.txt: ", s);
        }*/
        
        outputs = getTomcatOutputs("localhost_access_logs.txt");
        /*for(String s : outputs)
        {
            logInfo("Bad login localhost_access_logs.txt: ", s);
        }*/
        
        //copy back the good login info
        prepareWebStorageConfigFiles();
    }
    
    public static void main(String[] args) {
        new AgentWebStorageTest().doTests(args);
    }

}
